﻿/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 /*
  * $Created by: XIAO Tong (xiaotong@mail.neu.edu.cn) 2018-07-10
  */

#include <stdio.h>
#include "XNet.h"
#include "../tensor/XUtility.h"
#include "../tensor/function/FHeader.h"
#include "../tensor/core/CHeader.h"
#include "../sample/transformer/Transformer.h"
#include <fstream>
#include <string>

using namespace nts;
using namespace transformer;

void test() {
    XTensor posEmbeddingBase;
    int length = 5;
    int eSize = 4;
    int d = 4;
    InitTensor2D(&posEmbeddingBase, length, eSize, X_FLOAT);

    float* data = new float[posEmbeddingBase.unitNum];

    for (int pos = 0; pos < length; pos++) {
        float* dp = data + pos * eSize;

        //int channelSize = eSize / 2;
        //int offset = 0;
        //for(int i = 0; i < channelSize; i++){
        //    dp[offset++] = (float)sin(pos/pow(10000.0F, 2.0F*i/(d - 2)));
        //}
        //for(int i = 0; i < channelSize; i++){
        //    dp[offset++] = (float)cos(pos/pow(10000.0F, 2.0F*i/(d - 2)));
        //}


        for (int k = 0; k < eSize; k++) {
            if (k % 2 == 0) {
                int i = k / 2;
                dp[k] = (float)sin(pos / pow(10000.0F, 2.0F * i / d));
            }
            else {
                int i = (k - 1) / 2;
                dp[k] = (float)cos(pos / pow(10000.0F, 2.0F * i / d));
            }
        }

    }

    posEmbeddingBase.SetData(data, posEmbeddingBase.unitNum);
    posEmbeddingBase.Dump(stderr);
    delete[] data;
}

struct A {
    XTensor a;

    void update(XTensor b) {
        a = b;
    }
};

void test2(A *a) {
    XTensor x;
    InitTensor2D(&x, 2, 3);
    XTensor y;
    InitTensor2D(&y, 3, 2);
    float data[]{ 1,1,1,1,1,1 };
    x.SetData(data, 6);
    y.SetData(data, 6);
    XTensor z;
    z = MatrixMul(x, y);
    a->update(z);
}

void TestMemory() {
    int devID = 0;
    int memSize = 1024;
    XMem *mem = new XMem(devID, FREE_ON_THE_FLY, (MTYPE)MILLION * 256, 1024, MILLION * 128);
    mem->SetDesiredSize(devID, 0, (MTYPE)memSize * MILLION);
    XTensor a;
    InitTensor2D(&a, 5, 5, X_FLOAT, 0, mem);
    float d[25]{ 0 };
    for (int i = 0; i < 25; i++)
        d[i] = float(i);
    a.SetData(d, 25);

    int index[]{ 0,1,2,3,4 };
    for (int i = 0; i < 4; i++) {
        XTensor srcIdx, tgtIdx;
        InitTensor1D(&srcIdx, 4 - i, X_INT, a.devID, a.mem);
        InitTensor1D(&tgtIdx, 4 - i, X_INT, a.devID, a.mem);
        srcIdx.SetData(index, srcIdx.unitNum);
        tgtIdx.SetAscendingOrder(0);

        a = CopyIndexed(a, 0, srcIdx, tgtIdx);
        printf("\nround %d\n", i);
        a.Dump(stderr);
    }

    delete mem;
}

int main(int argc, const char** argv)
{

    TransformerMain(argc - 1, argv + 1);
        

    return 0;
}

