/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-04-03
*/

#ifndef __DATASET_H__
#define __DATASET_H__

#include <cstdio>
#include "../../..//tensor/XTensor.h"
#include "../../..//tensor/XGlobal.h"

namespace nts {


/* `DataSet` maintains data buffers for the inference stage .*/
struct DataSet {

public:

    /* the data buffer */
    ExampleList srcBuffer;

    /* the result buffer */
    ResultList resBuffer;

    /* the offset of all examples in the data */
    LongList offset;

    /* wether sort the dataset */
    bool sortBuffer;

    /* id for each example */
    size_t id;

    /* size of the data buffer */
    size_t bufferSize;

    /* size of used data in buffer */
    size_t bufferUsed;

    /* size of data in the src file */
    size_t exampleNumber;

    /* current index of the offset */
    size_t index;

    /* the pointer of the src file stream */
    FILE * fp;

public:

    /* check if the buffer is empty */
    bool IsEmpty();

    /* load data from a file to the buffer */
    void LoadDataToBuffer();

    /* initlization function */
    void Init(const char* fname, size_t myBufferSize, bool mySortBuffer);

    /* generate a mini-batch */
    IntList LoadBatch(XTensor * batchEnc, XTensor * paddingEnc, size_t batchSize, int devID);

    /* sort results by their ids */
    void SortRes();

    /* transform text file to binary file */
    static void ConvertFile(const char* src, const char* tgt);

    /* de-constructor */
    ~DataSet();
};

} // namespace nts(NiuTrans.Tensor)

#endif // __DATASET_H__