/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-03-18
*/

#include "StringUtil.h"

/* 
split string by delimiter, this will return indices of all sub-strings
>>> s - the original string
>>> delimiter - as it is
>>> a - the indices of all sub-strings
*/
void SplitToPos(const string& s, const string& delimiter, LongList& indices)
{
    if (delimiter.length() == 0) {
        indices.Add(0);
    }
    int pos = 0;
    int start = 0;
    while ((pos = s.find(delimiter, start)) != string::npos) {
        if (pos != start) {
            indices.Add(start);
        }
        start = pos + delimiter.length();
    }
    if (start != s.length()) {
        indices.Add(start);
    }
}

IntList SplitInt(const string& s, const string& delimiter)
{
    IntList fields;
    LongList indices;
    SplitToPos(s, delimiter, indices);
    for (int i = 0; i < indices.Size(); i++) {
        fields.Add(strtol(s.data() + indices[i], nullptr, 10));
    }
    return fields;
}

void SplitInt(const string& s, const string& delimiter, IntList& fields)
{
    LongList indices;
    SplitToPos(s, delimiter, indices);
    for (int i = 0; i < indices.Size(); i++) {
        fields.Add(strtol(s.data() + indices[i], nullptr, 10));
    }
}

FloatList SplitFloat(const string& s, const string& delimiter)
{
    FloatList fields;
    LongList indices;
    SplitToPos(s, delimiter, indices);
    for (int i = 0; i < indices.Size(); i++) {
        fields.Add(strtof(s.data() + indices[i], nullptr));
    }
    return fields;
}

void SplitInt(const string& s, const string& delimiter, FloatList& fields)
{
    LongList indices;
    SplitToPos(s, delimiter, indices);
    for (int i = 0; i < indices.Size(); i++) {
        fields.Add(strtof(s.data() + indices[i], nullptr));
    }
}